use warnings;
use strict;
use Test::More tests => 27;
use Test::Exception;

use POWER::Event;
use POWER::Event::IO;
use POWER::Utils::IO;

my $USUAL = '/tmp';
my $NETFS = '/mnt/Murzik_D';
my $TMPFS = '/dev/shm';

END { system("rm -f {\Q$USUAL\E,\Q$NETFS\E,\Q$TMPFS\E}/_type_.*.$$") }
my $E = $IN|$OUT;

my (@Event, $io);

sub reg {
    my ($type, $io) = @_;
    push @Event, [$type, $io->{events}, $io->{err}];
    die 'reg done';
}

# usual file

open my $wr, '>', "$USUAL/_type_.wr.$$" or die "open: $!";
print $wr 'wr' or die "print $!";
Nonblock($wr);
throws_ok { POWER::Event::IO->new({ cb => sub{reg('wr',@_)}, wait_for => $E,
    fh => $wr });   }           qr/Operation not permitted/;
close $wr or die "close: $!";
is(-s "$USUAL/_type_.wr.$$", 2, '> $wr');

system("echo rd > \Q$USUAL/_type_.rd.$$\E");
open my $rd, '<', "$USUAL/_type_.rd.$$" or die "open: $!";
sysread $rd, my $rd_buf, 1 or die "sysread: $!";
Nonblock($rd);
throws_ok { POWER::Event::IO->new({ cb => sub{reg('rd',@_)}, wait_for => $E,
    fh => $rd });   }           qr/Operation not permitted/;
close $rd;
is($rd_buf, 'r', '< $rd');

# file on network file system

SKIP: { skip 'no network fs available', 4 if !-w $NETFS;
                 
open my $Nwr, '>', "$NETFS/_type_.Nwr.$$" or die "open: $!";
print $Nwr 'Nwr' or die "print $!";
Nonblock($Nwr);
throws_ok { POWER::Event::IO->new({ cb => sub{reg('Nwr',@_)}, wait_for => $E,
    fh => $Nwr });   }           qr/Operation not permitted/;
close $Nwr or die "close: $!";
is(-s "$NETFS/_type_.Nwr.$$", 3, '> $Nwr');

system("echo Nrd > \Q$NETFS/_type_.Nrd.$$\E");
open my $Nrd, '<', "$NETFS/_type_.Nrd.$$" or die "open: $!";
sysread $Nrd, my $Nrd_buf, 1 or die "sysread: $!";
Nonblock($Nrd);
throws_ok { POWER::Event::IO->new({ cb => sub{reg('Nrd',@_)}, wait_for => $E,
    fh => $Nrd });   }           qr/Operation not permitted/;
close $Nrd;
is($Nrd_buf, 'N', '< $Nrd');

};

# file on tmpfs

open my $Twr, '>', "$TMPFS/_type_.Twr.$$" or die "open: $!";
print $Twr 'Twr' or die "print $!";
Nonblock($Twr);
throws_ok { POWER::Event::IO->new({ cb => sub{reg('Twr',@_)}, wait_for => $E,
    fh => $Twr });   }           qr/Operation not permitted/;
close $Twr or die "close: $!";
is(-s "$TMPFS/_type_.Twr.$$", 3, '> $Twr');

system("echo Trd > \Q$TMPFS/_type_.Trd.$$\E");
open my $Trd, '<', "$TMPFS/_type_.Trd.$$" or die "open: $!";
sysread $Trd, my $Trd_buf, 1 or die "sysread: $!";
Nonblock($Trd);
throws_ok { POWER::Event::IO->new({ cb => sub{reg('Trd',@_)}, wait_for => $E,
    fh => $Trd });   }           qr/Operation not permitted/;
close $Trd or die "close: $!";
is($Trd_buf, 'T', '< $Trd');

# file on procfs

open my $Prd, '<', "/proc/version" or die "open: $!";
sysread $Prd, my $Prd_buf, 5 or die "sysread: $!";
Nonblock($Prd);
throws_ok { POWER::Event::IO->new({ cb => sub{reg('Prd',@_)}, wait_for => $E,
    fh => $Prd });   }           qr/Operation not permitted/;
close $Prd or die "close: $!";
is($Prd_buf, 'Linux', '< $Prd');

# file on devfs(udev)

my ($Drd, $Drd_buf, $Dwr);

open $Drd, '<', "/dev/zero" or die "open: $!";
sysread $Drd, $Drd_buf, 1 or die "sysread: $!";
Nonblock($Drd);
throws_ok { POWER::Event::IO->new({ cb => sub{reg('Drd',@_)}, wait_for => $E,
    fh => $Drd });   }           qr/Operation not permitted/;
close $Drd or die "close: $!";
is($Drd_buf, "\x00", '< $Drd');

open $Drd, '<', "/dev/null" or die "open: $!";
defined(sysread $Drd, $Drd_buf, 1) or die "sysread: $!";
Nonblock($Drd);
throws_ok { POWER::Event::IO->new({ cb => sub{reg('Drd',@_)}, wait_for => $E,
    fh => $Drd });   }           qr/Operation not permitted/;
close $Drd or die "close: $!";
is($Drd_buf, "", '< $Drd');

open $Dwr, '>', "/dev/null" or die "open: $!";
print $Dwr 'Dwr' or die "print $!";
Nonblock($Dwr);
throws_ok { POWER::Event::IO->new({ cb => sub{reg('Dwr',@_)}, wait_for => $E,
    fh => $Dwr });   }           qr/Operation not permitted/;
close $Dwr or die "close: $!";
is(-s "/dev/null", 0, '> $Dwr');

open $Drd, '<', "/dev/random" or die "open: $!";
sysread $Drd, $Drd_buf, 1 or die "sysread: $!";
Nonblock($Drd);
lives_ok { $io=POWER::Event::IO->new({ cb => sub{reg('Drd',@_)}, wait_for => $E,
    fh => $Drd });   } '... /dev/random pollable';
eval { loop() }; die "in loop: $@" if $@ !~ /reg done/;
is_deeply(shift @Event, ['Drd',$IN,undef], '    got $IN');
close $Drd or die "close: $!";
$io->close();
like($Drd_buf, qr/./xms, '< $Drd');

open $Drd, '<', "/dev/urandom" or die "open: $!";
sysread $Drd, $Drd_buf, 1 or die "sysread: $!";
Nonblock($Drd);
throws_ok { POWER::Event::IO->new({ cb => sub{reg('Drd',@_)}, wait_for => $E,
    fh => $Drd });   }          qr/Operation not permitted/;
close $Drd or die "close: $!";
like($Drd_buf, qr/./xms, '< $Drd');

# directory

mkdir "$USUAL/_type_.dir.$$" or die "mkdir: $!";
opendir my $DIR, "$USUAL/_type_.dir.$$" or die "opendir: $!";
defined(my $DIR_buf = readdir $DIR) or die "readdir: $!";
is(fileno($DIR), undef, 'no fileno(DIRHANDLE)');
closedir $DIR or die "closedir: $!";
like($DIR_buf, qr/\./xms, '< $DIR');
rmdir "$USUAL/_type_.dir.$$" or die "rmdir: $!";

