use warnings;
use strict;
use Test::More 'no_plan';     # depends on system buffers :(
use Test::Exception;
use Test::MockModule;

use Socket;

use POWER::Event;
use POWER::Event::IO;
use POWER::Utils::IO;


my $SIZE = 204800;


POWER::Event::IO->new({
    type        => $LISTEN,
    host        => '0.0.0.0',
    port        => 1234,
    cb          => \&listener,
    wait_for    => $ACCEPTED,
});
my $io = POWER::Event::IO->new({
    fh          => tcp_client('127.0.0.1', 1234),
    state       => $CONNECTING,
    cb          => 'Client',
    method      => 'IO_client',
    wait_for    => $OUT,
    out_buf     => 'x' x $SIZE,
    out_pos     => 0,
    Prev_pos    => 0,
});
is(fileno($io->{fh}), $io->{fd}, 'fileno() == {fd}');

eval { loop() };
die "out of loop: $@";

sub listener {
    my ($io) = @_;
    die "listener unexpected event: $io->{err}" if $io->{events} != $ACCEPTED;
    my($port,$iaddr) = sockaddr_in($io->{in_paddr});
    my $ip = inet_ntoa($iaddr);
    is($ip, '127.0.0.1', 'ip correct');
    ok($port > 32000,    'port looks good');
    POWER::Event::IO->new({
        fh          => delete($io->{in_buf}),
        cb          => 'Server',
        wait_for    => $IN,
        in_buf_limit=> 1024,
        Prev_bytes  => 0,
        LimitErrs   => 0,
    });
}

package Server;
use Test::More;
use POWER::Event::IO;
sub IO {
    my ($self, $io) = @_;
    if ($io->{events} & $ERR && $io->{err} =~ /in_buf_limit/) {
        $io->{LimitErrs}++;
        ok($io->{LimitErrs}, 'got <in_buf_limit reached> error');
        if ($io->{LimitErrs} == 5) {
            $io->{in_buf_limit}=0;
        }
        $io->{events} &= ~$ERR;
        return unless $io->{events};
    }
    die "server unexpected event: $io->{err}" if $io->{events} != $IN;
    if (!$io->{is_eof}) {
        ok($io->{in_bytes} > $io->{Prev_bytes},    '  in_bytes incremented, good');
        is(length($io->{in_buf}), $io->{in_bytes}, '  in_bytes correct');
    }
    $io->{Prev_bytes} = $io->{in_bytes};
    if ($io->{in_bytes} < $SIZE) {
        ok(!$io->{is_eof}, 'no eof yet');
    } else {
        if (!$io->{All_received}) {
            is($io->{in_bytes}, $SIZE, 'All data received');
            $io->{All_received} = 1;
        } else {
            ok($io->{is_eof}, 'now got {is_eof} without $EOF!!!');
            exit;
        }
    }
}

package Client;
use Test::More;
use POWER::Event::IO;
sub IO_client {
    my ($self, $io) = @_;
    die "server unexpected event: $io->{err}" if $io->{events} != $OUT;
    ok($io->{out_pos} > $io->{Prev_pos}, '  some bytes was sent');
    is(length($io->{out_buf}), $SIZE,    '  out_buf does not changed, good');
    $io->{Prev_pos} = $io->{out_pos};
    shutdown $io->{fh}, 1 if $io->{out_pos} == $SIZE;
}

